extern crate uinput;
extern crate evdev_rs;
extern crate crossbeam;

use std::env::args;
use std::time::Duration;
use std::collections::HashMap;

use evdev_rs::enums::EventType::{EV_KEY, EV_REL};

use crossbeam::crossbeam_channel:: Select;

mod virtual_mouse;
mod virtual_kbd;
mod hw_device;
mod event;
mod input_thread;

use self::input_thread::create_evdev_thread;


fn main() {
    let args: Vec<_> = args().collect();

    // (path, receiver, stopper)
    let mut devices = Vec::new();
    // let mut disabled_devices = Vec::new();

    for device_path in &args[1..] {
        devices.push(create_evdev_thread(&device_path));
    }

    // testing device removal
    // thread::sleep(Duration::from_secs(1));
    // devices[1].2.send(true).unwrap();
    // disabled_devices.push(devices.remove(1));

    // init virtual devices
    let mut virt_kbd = virtual_kbd::VirtualKeyboard::new("test-kbd");
    let mut virt_mouse = virtual_mouse::VirtualMouse::new("test-mouse");

    // read events and feed them to the virtual devices
    loop {
        // init select
        let mut sel = Select::new();

        // create device input event receive operations and map them to devices
        let mut device_idx_by_oper = HashMap::new();
        for (i, (_, receiver, _)) in devices.iter().enumerate() {
            let o = sel.recv(&receiver);
            device_idx_by_oper.insert(o, i);
        }

        // select from available device(s)
        let oper = sel.select();

        // read from selected device
        let index = oper.index();
        match oper.recv(&devices[device_idx_by_oper[&index]].1) {
            Ok(e) => {
                // mouse event
                if e.typ == EV_REL as u32 || (e.code >= 272 && e.code <= 276) {
                    virt_mouse.device.write(e.typ as i32, e.code as i32, e.value).unwrap();
                    virt_mouse.device.synchronize().unwrap();
                // keyboard event
                } else if e.typ == EV_KEY as u32 {
                    virt_kbd.device.write(e.typ as i32, e.code as i32, e.value).unwrap();
                    virt_kbd.device.synchronize().unwrap();
                }
            },
            _ => (),
        }
    }
}
